#!/usr/local/bin/perl

# model2.pl version 0.01
# Program to takes parallel text and determines the alignments
# and their probability for each sentence based on IBM Model2
#
# Copyright (C) 2002-2003
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Bridget McInnes, University of Minnesota, Duluth
# bthomson@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

#we have to use commandline options, so use the necessary package
use Getopt::Long;

#for LINUX because we are possible using french characters
use bytes;

#first check if no commandline options have been provided... in which case
#print out the usage notes
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

#now get the options
GetOptions(  "version", "help", "sourcevcb=s", "targetvcb=s", "matrixfile=s", "distortfile=s", "sntfile=s", "converge=f", "iterate=i", "precision=i", "alignmentfile=s", "translationfile=s", "wordalignmentfile=s", "$distortion=i" );

#if help has been requested, print out help
if ( defined $opt_help )
{
  $opt_help = 1;
  &showHelp();
  exit;
}

#if version has been requested, show version
if ( defined $opt_version )
{
  $opt_version = 1;
  &showVersion();
  exit;
}

#open the snt file or die
open(SNT, $opt_sntfile) || die "SNT file  not found.\n";

#open the sourcevcb file or die
if ( defined $opt_sourcevcb) {
  open(SRCVCB, $opt_sourcevcb) || die "$opt_sourcevcb file not found.\n";
}
else { print "Source vocabulary file not defined.\n"; exit; }

#open the targetvcb file or die
if( defined $opt_targetvcb) {
  open(TRGVCB, $opt_targetvcb) || die "$opt_targetvcb file not found.\n";
}
else { print "Target vocabulary file not defined.\n"; exit;}

#open the adjacency list and reverse distortion file or die
if(defined $opt_matrixfile) { $matrixfile = $opt_matrixfile; }
else { $matrixfile = $opt_sntfile . ".g"; }

if ( defined $opt_distortfile ) { $distortfile = $opt_distortfile; }
else {  $distortfile = $opt_sntfile . ".d"; }

open(MATRIX, $matrixfile)   || die "Adjacency list file not found.\n";
open(DISTORT, $distortfile) || die "Reverse distortion file not found.\n"; 

#set the translation, alignment and wordalignment files
$translationfile = defined $opt_translationfile ? ">" . $opt_translationfile : ">" . $opt_sntfile . ".translations";

$alignmentfile = defined $opt_alignmentfile ? ">" . $opt_alignmentfile : ">" . $opt_sntfile . ".alignments";

$wordalignmentfile = defined $opt_wordalignmentfile ? ">" . $opt_wordalignmentfile : ">" . $opt_sntfile . ".wa";

open(TRNS, $translationfile) || die "could not open the translation file";
open(ALGN, $alignmentfile)   || die "could not open the alignment file";
open(WA, $wordalignmentfile) || die "could not open the word alignment file";

#if converging factor has been defined set the convergence
if( defined $opt_converge ) { $converge = $opt_converge; }

if($converge > 1 || $converge < 0) {
  print "Converge must be a float between zero and one.\n";
  exit;
}

#if iteration factor has been defined set interactions
$iterate = defined $opt_iterate ? $opt_iterate : 4;

if($iterate < 0) {
  print "Iterate must be an integer greater than one.\n";
  exit;
}

#if the distortion factor as been defined set else set to default of 2
$distortion = defined $opt_distortion ? $opt_distortion : 2;

#if precision has been defined set precision to it else set it 
#to a default of four
if(defined $opt_precision) {
  $precision = 1;
  for($i = 0; $i<$opt_precision; $i++) {
    $precision = $precision . "0";
  }
}else { $precision = 10000; }

# add the database
 use Fcntl;
use NDBM_File;
use vars qw( %model$k $v );

unlink "model";

tie %model, "NDBM_File", "model", O_RDWR|O_CREAT, 0640
  or die "Error can not open file 'freq' : $!\n";

# add the database


my %tp=();
my %model = ();
my %sentencelengths = ();
my $count = 0;

print "Loading Adjacency List ...\n";
while(<MATRIX>) {
  chomp;

  if($_ == 1) {
    $count++;
    $sourcelength = <MATRIX>;
    $targetlength = <MATRIX>;

    chomp $sourcelength;
    chomp $targetlength;

    $sentencelengths{$count} = $sourcelength . " " . $targetlength;
  }

  @matrix = split/\s+/, $_; 
  $source = shift @matrix;

  ($s, $spos) = split/\//, $source;
  foreach $target (@matrix) {
    ($t, $tpos) = split/\//, $target;
    $model{$count}{$target}{$source}++;
    $tp{$s}{$t}++;
  }
}
close(MATRIX);
print "Adjacency List Loaded.\n";

#set the alignment probabilities
print "Load Alignment Probabilities File...\n";
%ap = ();
while(<DISTORT>) {
  chomp;
  @distort = split/\s+/, $_;

  $tmp = $distort[1] . " " . $distort[2] . " " . $distort[3];
  $ap{$distort[0]}{$tmp} = $distort[4];
}
print "Alignment Probabilities File Loaded.\n";

#Set the initial translation probabilities
foreach $s ( keys %tp ) {
  @array = keys %{ $tp{$s} };
  foreach $t ( keys %{ $tp{$s} } ) {
    $tp{$s}{$t} = 1/($#array+1);
  }
}

my $check = "false";
my $convergence = "false";
my $iterations = 0;

#begin em while loop
while($check eq "false") {

  print "\nITERATION $iterations\n";
  print "-------------------------------------------\n";
  #compute P(a,f|e) and aE P(a,f|e) foreach sentence
  %tc = ();
  %ac = ();
  foreach $count ( keys %model ) {
    ($slength, $tlength) = split/\s+/, $sentencelengths{$count};
    foreach $target ( keys %{ $model{$count} } ) {
      ($t, $tpos) = split/\//, $target;
      $total = 0;
      $tmp = $tpos . " " . $slength . " " . $tlength;
      foreach $source  ( keys %{ $model{$count}{$target} } ) { 
	($s, $spos) = split/\//, $source;
	$total += $tp{$s}{$t} * $ap{$spos}{$tmp}; 
      }

      foreach $source (keys %{ $model{$count}{$target} } ) {
	($s, $spos) = split/\//, $source;
	if(exists $tc{$s}{$t}) {       $tc{$s}{$t} += $tp{$s}{$t} * $ap{$spos}{$tmp} / $total;      }
	else {                         $tc{$s}{$t}  = $tp{$s}{$t} * $ap{$spos}{$tmp} / $total;      }

	if(exists $ac{$spos}{$tmp}) {  $ac{$spos}{$tmp} += $tp{$s}{$t} * $ap{$spos}{$tmp} / $total; }
	else {                         $ac{$spos}{$tmp}  = $tp{$s}{$t} * $ap{$spos}{$tmp} / $total; }	
      }
    }
  }

  #normalize the translational probabilities
  foreach $s ( keys %tc ) {
    $normalize = 0;
    foreach $t ( keys %{ $tc{$s} } ) {
      $normalize += $tc{$s}{$t};
    }
    foreach $t ( keys %{ $tc{$s} } ) {
      if(defined $opt_converge) {
	$factor = abs($tp{$s}{$t} - ($tc{$s}{$t}/$normalize));
	$convergence = $factor > $opt_converge ? "false" : "true";
      }
      $tp{$s}{$t} = $tc{$s}{$t}/$normalize;
    }
  }

  #normalize the alignment probabilities
  foreach $spos ( keys %ac ) {
    $normalize = 0;
    foreach $tmp ( keys %{ $ac{$spos} } ) {
      $normalize += $ac{$spos}{$tmp};
    }
    foreach $tmp ( keys %{ $ac{$spos} } ) {
      $ap{$spos}{$tmp} = $ac{$spos}{$tmp} / $normalize;
    }
  }

  #increment iterations
  $iterations++;

  #if iterations are less then the defined iterate then
  #continue otherwise stop
  if(!defined $opt_converge) {
    $check = $iterations < $iterate ? "false" : "true";
  }
  else {
    $check = $convergence eq "false" ? "false" : "true";
  }
} #end em while loop

#untie the database
untie %model;

print "Loading Vocabulary Files ...\n";
my %srcvcb = ();
my %trgvcb = ();
while(<SRCVCB>) {
    chomp;
    ($num, $trans) = split/\s+/, $_;
    $srcvcb{$num} = $trans;
}
close SRCVCB;
while(<TRGVCB>) {
    chomp;
    ($num, $trans) = split/\s+/, $_;
    $trgvcb{$num} = $trans;
}
close TRGVCB;
print "Finished Loading Vocabulary Files\n";


#Load translation file
#probabilities dumped as source target prob
foreach $s (sort  keys %tp) {
    foreach $t ( sort keys %{ $tp{$s} }) {
	if($t eq " ") { print TRNS "srcvcb{$s} NULL $tp{$s}{$t}\n"; }
	else { print TRNS "$srcvcb{$s} $trgvcb{$t} $tp{$s}{$t}\n"; }
    }	
}

#Load alignmentfile
foreach $spos ( keys %ac ) {
    foreach $tmp ( keys %{ $ac{$spos} } ) {
	print ALGN "$spos $tmp $ac{$spos}{$tmp}\n";
    }
}

#get alignments for each sentences and print to the 
#word alignment file

$count = 0;
while(<SNT>) {
  if($_ == 1) {
    $sourceline = <SNT>;
    $targetline = <SNT>;

    chomp $sourceline;
    chomp $targetline;

    @source = split/\s+/,$sourceline;
    @target = split/\s+/,$targetline;
	
    $sourcelength = $#source+1;
    $targetlength = $#target+1;
	
    print WA "# Sentence pair ($count) ";
    print WA "source length $sourcelength ";
    print WA "target length $stargetlength ";

    $alignmentscore = 1;
    @targetsentence = ();
	
    unshift @source, 0;
	
    my %final = ();
    for $i(0..$#target) {
      $maxprob = -1;
      $start = ($i - $distortion < 0) ?  0 : $i - $distortion;
      $end = ($i + $distortion > $#source) ? $#source : $i + $distortion;
      $tpos = $i;
      $spos = $i;

      for ($j = $end; $j >= $start; $j--) { 
	if($maxprob < $tp{$source[$j]}{$target[$i]}) {
	  $maxprob = $tp{$source[$j]}{$target[$i]};
	  $final{$target[$i]."/".$i} = $j;
	  $tpos = $j;
	}
	if($maxprob == $tp{$source[$j]}{$target[$i]} && $n[$j+1] == 0) {
	  $maxprob = $tp{$source[$j]}{$target[$i]};
	  $final{$target[$i]."/".$i} = $j;
	  $tpos = $j;
	}
      }
      $alignmentscore *= $maxprob;
    }
    $n[$tpos] = 1;
    print WA "alignment score : $alignmentscore \n";

    for $i (1..$#source+1) {
      print WA "$srcvcb{$source[$i]} ";
    }
    print WA "\n";
    print WA "NULL ({ }) ";
    for $j (0..$#target) { 
      $place = $final{$target[$j]."/".$j};
      print WA "$trgvcb{$target[$j]} ({$place}) ";
      $spos = $j;
    }
    print WA "\n";
  }
  $count++;
}


sub minimalUsageNotes
{
  print STDERR "Usage: model2.pl [OPTIONS] --sourcevcb FILE --targetvcb FILE --sntfile FILE \n";
  askHelp();
}
# function to output help messages for this program
sub showHelp 
{
  print "\nUsage: model2.pl [OPTIONS] --targetvcb FILE  --sourcevcb FILE --snfile FILE \n\n";

  print "OPTIONS:\n\n";

  print "  --alignmentfile     FILE   File to store postiional alignments of the\n";
  print "                             sentences.\n";
  print "                             Default : <SNTFILE>.alignments\n\n";

  print "  --translationfile   FILE   File to store possible word translations.\n";
  print "                             Default : <SNTFILE>.translations.\n\n";

  print "  --wordalignmentfile FILE   File that contains the word alignments of \n";
  print "                             parallel corpora. Default: <SNTFILE>.wa\n\n";

  print "  --distortfile       FILE   File that contains the distortion parameters.\n";
  print "                             Default: <SNTFILE>.d\n\n";

  print "  --matrixfile        FILE   File that contains the matrix parameters.\n";
  print "                             Default: <SNTFILE>.g\n\n";

  print "  --converge          FLOAT  Stop iterating when the difference\n";
  print "                             between values is less than this value.\n";
  print "                             Note: 0 > FLOAT > 1\n\n";

  print "  --iterate           INT    Stop iterating at a specified number of\n";
  print "                             iterations. \n";
  print "                             Note: INT > 0  Default : 10\n\n";

  print "  --precision         INT    How many digits of precision in displaying\n";
  print "                             the output in the translation file and the\n";
  print "                             alignment file   Default : 5\n\n";

  print "  --version                  Prints the version number.\n\n";

  print "  --help                     Prints this help message.\n\n";

  print "REQUIRED:\n\n";

  print "  --sourcevcb  FILE        Name of source vocabulary file.\n\n";

  print "  --targetvcb  FILE        Name of target vocabulary file.\n\n";     

  print "  --sntfile    FILE        Name of the snt file.\n\n";
}

#function to output the version number
sub showVersion
{
  print STDERR "Copyright (C) 2003, Ted Pedersen & Bridget McInnes\n";
  print STDERR "model2.pl  -  Version 0.01\n";
  print STDERR "11 March 2003\n";
}

#function to output "ask for help" message when the user's goofed up!
sub askHelp
{
  print STDERR "Type model2.pl --help for help.\n";
}

