#!/usr/local/bin/perl
# plain2snt.pl version 0.01
# Program to takes in sentence aligned parallel text and converts
# the text to numbered sentences and creates a vocabulary file for
# each text.
#
# Copyright (C) 2002-2003
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Bridget McInnes, University of Minnesota, Duluth
# bthomson@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#we have to use commandline options, so use the necessary package!
use Getopt::Long;
use bytes;

#first check if no commandline options have been provided... in which case
#print out the usage notes!
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

#now get the options!
GetOptions(  "version", "help", "source=s", "target=s", "sntfile=s", "sourcevcb=s", "targetvcb=s");

#if help has been requested, print out help!
if ( defined $opt_help )
{
    $opt_help = 1;
    &showHelp();
    exit;
}

#if version has been requested, show version!
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

#check if the target and source file have been defined
if ( defined $opt_target ) { $targetfile = $opt_target; }
if ( defined $opt_source ) { $sourcefile = $opt_source; }
#open the source and target files or die
open(TARGET, $targetfile) || die "File $targetfile not found.\n";
open(SOURCE, $sourcefile) || die "File $sourcefile not found.\n";

#prepare and open the output file
if ( defined $opt_sntfile ) { $output = ">" . $opt_sntfile; }
else { $output = ">" . $sourcefile . $targetfile . ".snt"; }

#prepare source vocab file
if ( defined $opt_sourcevcb ) { $sourcevcb = ">" . $opt_sourcevcb; }
else { $sourcevcb = ">" . $sourcefile . ".vcb"; }

#prepare target vocab file
if ( defined $opt_targetvcb ) { $targetvcb = ">" . $opt_targetvcb; }
else { $targetvcb = ">" . $targetfile . ".vcb"; }

#declare variables
%sourcehash = ();
%targethash = ();

$sourcecount = 2;
$targetcount = 2;

open(OUT, $output);

#create sentence file
while(<SOURCE>) {
    
    print OUT "\n1\n";

    chomp;
    $targetline = <TARGET>;
    chomp $targetline;

    @source = split /\s+/, $_;
    @target = split /\s+/, $targetline;

    foreach $word (@source) {
	if(exists $sourcehash{$word}) { 
	    print OUT "$sourcehash{$word} "; 
	}
	else { 
	    $sourcehash{$word} = $sourcecount;
	    print OUT "$sourcecount ";
	    $sourcecount++;
	}
    }
   
    print OUT "\n";
    foreach $word (@target) {
	if(exists $targethash{$word}) { 
	    print OUT "$targethash{$word} "; 
	}
	else { 
	    $targethash{$word} = $targetcount;
	    print OUT "$targetcount ";
	    $targetcount++;
	}
    }
}
	
close(OUT);
close(SOURCE);
close(TARGET);

#open vocab files
open(SRC, $sourcevcb);
open(TRG, $targetvcb);

foreach $set (sort { $sourcehash{$a} <=> $sourcehash{$b} } keys %sourcehash) {
    print SRC "$sourcehash{$set} $set\n";
}
foreach $set (sort { $targethash{$a} <=> $targethash{$b} } keys %targethash) {
    print TRG "$targethash{$set} $set\n";
}

close(SRC);
close(TRG);

#function to output the version number
sub showVersion
{
    print STDERR "Copyright (C) 2003, Ted Pedersen & Bridget McInnes\n";
    print STDERR "plain2snt.pl  -  Version 0.01\n";
    print STDERR "20 February 2003\n";
}

#function to output "ask for help" message when the user's goofed up!
sub askHelp
{ 
    print STDERR "Type plain2snt.pl --help for help.\n";
}

#function to output the usage message when the user's goofed up
sub minimalUsageNotes
{
    print STDERR "Usage: plain2snt.pl [OPTIONS] --target FILE --source FILE\n";
    askHelp();
}

#function to output help messages for this program
sub showHelp
{
    print "Usage:  plain2snt.pl [OPTIONS] --target FILE --source FILE\n";
    
    print "OPTIONS:\n\n";
    
    print "  --sntfile      FILE         File to store numbered sentences.\n ";
    print "                              Default: <SOURCE><TARGET>.snt\n\n";

    print "  --sourcevcb    FILE         File to store the source vocabulary \n";
    print "                              data. Default: <SOURCE>.vcb\n\n";

    print "  --targetvcb    FILE         File to store the target vocabulary \n";
    print "                              data. Default: <TARGET>\n\n";

    print "  --version                   Prints the version number.\n\n";
    
    print "  --help                      Prints this help message.\n\n";
    
    print "REQUIRED:\n\n";
    
    print "  --source       FILE         Name of source file.\n\n";
  
    print "  --target       FILE         Name of target file.\n\n";      
}
 
