#!/usr/local/bin/perl

# snt2matrix.pl version 0.01
# Program to takes the sntfile created by plain2snt.pl and creates
# an adjacency list based on the distortion factor.
#
# Copyright (C) 2002-2003
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Bridget McInnes, University of Minnesota, Duluth
# bthomson@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

#we have to use commandline options, so use the necessary package!
use Getopt::Long;


#first check if no commandline options have been provided... in which case
#print out the usage notes!
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

#now get the options!
GetOptions(  "version", "help", "sntfile=s", "matrixfile=s", "distortfile=s", "distortion=i");

#if help has been requested, print out help!
if ( defined $opt_help )
{
    $opt_help = 1;
    &showHelp();
    exit;
}

#if version has been requested, show version!
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

#check if input file has been defined
if( defined $opt_sntfile) { $input = $opt_sntfile; }
open(INPUT, $input) || die "File sntfile not found.\n";

#set output file and open
$outputfile = defined $opt_matrixfile ? ">" . $opt_matrixfile: ">" . $input . ".g";
open(OUTPUT, $outputfile);

#set distortion file
$distortfile = defined $opt_distortfile ? ">" . $opt_distortfile: ">" . $input . ".d";
open(DISTORT, $distortfile);

#if distortion factor has been defined set distort 
$distort = defined $opt_distortion ? $opt_distortion : 2;

print "DISTORTION => $distort\n";

#check distortion size
if($distort < 0) { 
    print "Distortion must be an integer greater than zero.\n";
    exit;
}

#initialize hash tables
%a = ();
%y = ();

while(<INPUT>) {
    chomp;
    if($_ == 1) {

	print OUTPUT "$_\n";


	$sourcesentence = <INPUT>;
	$targetsentence = <INPUT>;

	chomp $targetsentence;
	chomp $sourcesentence;


	@source = split/\s+/, $sourcesentence;
	@target = split/\s+/, $targetsentence;
	

	unshift @source, 0;
	unshift @target, 0;
	

	$sourcelength = $#source;
	$targetlength = $#target;

	print OUTPUT "$sourcelength\n";
	print OUTPUT "$targetlength\n";
			       
	for $i (1..$#source) {
	    print OUTPUT "$source[$i]/$i ";
	    if($i <= $#target) { 
		print OUTPUT " $target[$i]/$i"; 
		$temp1 =  $i . " " . $i . " " . $sourcelength . " " . $targetlength;
		$temp2 = $i . " " . $sourcelength . " " . $targetlength;
		$a{$temp1}++;
		$y{$temp2}++;
	    }

	    $plus = $i+1;
	    $minus = $i-1;

	    for $j (1..$distort) {
		if($plus <= $#target) { 
		    print OUTPUT " $target[$plus]/$plus"; 
		    $temp1 =  $i . " " . $plus . " " . $sourcelength . " " . $targetlength;
		    $temp2 = $plus . " " . $sourcelength . " " . $targetlength;
		    $a{$temp1}++;
		    $y{$temp2}++;
		}
		if($minus > 0) { 
		    print OUTPUT " $target[$minus]/$minus"; 
		    $temp1 =  $i . " " . $minus . " " . $sourcelength . " " . $targetlength;
		    $temp2 = $minus . " " . $sourcelength . " " . $targetlength;
		    $a{$temp1}++;
		    $y{$temp2}++; 
		}
		$plus++; 
		$minus--; 
	    } 
	    print OUTPUT  "\n";
	}  
    }  
}  

close OUTPUT;

foreach $item (keys %a) {
    ($spos, $tpos, $slength, $tlength) = split/\s+/, $item;
    $den = $tpos . " " . $slength . " " . $tlength;
    $prob = $a{$item}/$y{$den};
    print DISTORT "$item $prob\n";
}
	    
close DISTORT;
#function to output the version number
sub showVersion
{
    print STDERR "Copyright (C) 2003, Ted Pedersen & Bridget McInnes\n";
    print STDERR "snt2matrix.pl  -  Version 0.01\n";
    print STDERR "29 February 2003\n";
}

#function to output "ask for help" message when the user's goofed up!
sub askHelp
{ 
    print STDERR "Type snt2matrix.pl --help for help.\n";
}

#function to output the usage message when the user's goofed up
sub minimalUsageNotes
{
    print STDERR "Usage: snt2matrix.pl [OPTIONS] --sntfile FILE \n";
    askHelp();
}

#function to output help messages for this program
sub showHelp
{
    print "\nUsage:  snt2matrix.pl [OPTIONS] --sntfile FILE\n\n";
    
    print "OPTIONS:\n\n";
    
    print "  --matrixfile  FILE         File to store adjacency list.\n";
    print "                             Default: <SNTFILE>.g\n\n";

    print "  --distortfile FILE         File to store reverse distortion\n";
    print "                             results. Default: <SNTFILE>.d\n\n";

    print "  --distortion  INT          Sets the distortion factor. \n";
    print "                             Default: distortion = 2\n\n";

    print "  --version                  Prints the version number.\n\n";
    
    print "  --help                     Prints this help message.\n\n";
    
    print "REQUIRED:\n\n";
    
    print "  --sntfile      FILE        Name of sntfile file created";
    print "                             by plain2snt.pl \n";
}
 
